@@ -8,36 +8,36 @@ from cStringIO import StringIO
 
 
 if __name__ == '__main__':
-    from __init__ import bitarray, bits2bytes, _getIndicesEx
+    from __init__ import bitarray, bits2bytes
     repr_type = "<class '__init__.bitarray'>"
 else:
-    from bitarray import bitarray, bits2bytes, _getIndicesEx
+    from bitarray import bitarray, bits2bytes
     repr_type = "<class 'bitarray.bitarray'>"
 
 
 tests = []
 
 
-class Util():
+class Util(object):
     
-    def randombitarrays(self, N=20):
-        for n in xrange(N):
+    def randombitarrays(self):
+        for n in range(25) + [randint(1000, 2000)]:
             yield bitarray([randint(0, 1) for d in xrange(n)],
                            endian='big' if randint(0, 1) else 'little')
 
-    def randomlists(self, N=20):
-        for n in xrange(N):
+    def randomlists(self, N=500):
+        for n in range(25) + [randint(1000, 2000)]:
             yield [bool(randint(0, 1)) for d in xrange(n)]
 
     def rndsliceidx(self, length):
         return randint(-2*length, 2*length-1) if randint(0, 1) == 1 else None
 
     def slicelen(self, r, length):
-        return _getIndicesEx(r, length)[-1]
+        return getIndicesEx(r, length)[-1]
 
     def check_obj(self, a):
         self.assertEqual(repr(type(a)), repr_type)
-        unused = 8 * a.buffer_info()[1] - a.length()
+        unused = 8 * a.buffer_info()[1] - len(a)
         self.assert_(0 <= unused < 8)
         self.assertEqual(unused, a.buffer_info()[3])
 
@@ -47,6 +47,48 @@ class Util():
         self.check_obj(a)
         self.check_obj(b)
 
+
+def getIndicesEx(r, length):
+    if not isinstance(r, slice):
+        raise TypeError("slice object expected")
+    start = r.start
+    stop  = r.stop
+    step  = r.step
+    if r.step is None:
+        step = 1
+    else:
+        if step == 0:
+            raise ValueError("slice step cannot be zero")
+    
+    defstart = length-1 if step < 0 else 0
+    defstop  = -1 if step < 0 else length
+    
+    if r.start is None:
+        start = defstart
+    else:
+        if start < 0: start += length
+        if start < 0: start = -1 if step < 0 else 0
+        if start >= length: start = length-1 if step < 0 else length
+    
+    if r.stop is None:
+        stop = defstop
+    else:
+        if stop < 0: stop += length
+        if stop < 0: stop = -1
+        if stop > length: stop = length
+
+    if (step < 0 and stop >= length) or (step > 0 and start >= stop):
+        slicelength = 0
+    elif step < 0:
+        slicelength = (stop-start+1) / step + 1
+    else:
+        slicelength = (stop-start-1) / step + 1
+
+    if slicelength < 0:
+        slicelength = 0
+
+    return start, stop, step, slicelength
+
 # ---------------------------------------------------------------------------
 
 class TestsModuleFunctions(unittest.TestCase, Util):
@@ -69,7 +111,8 @@ class TestsModuleFunctions(unittest.TestCase, Util):
                      (10, 2), (15, 2), (16, 2), (64, 8), (65, 9),
                      (0L, 0), (1L, 1), (65L, 9), (2**29, 2**26),
                      (2**31, 2**28), (2**32, 2**29), (2**34, 2**31),
-                     (2**34+793, 2**31+100), (2**35-8, 2**32-1)]:
+                     (2**34+793, 2**31+100), (2**35-8, 2**32-1),
+                     (2**62, 2**59), (2**63-8, 2**60-1)]:
             self.assertEqual(bits2bytes(n), m)
 
 
@@ -85,7 +128,6 @@ class CreateObjectTests(unittest.TestCase, Util):
         self.assertEqual(a.tolist(), [])
         self.check_obj(a)
 
-
     def test_endian(self):
         a = bitarray('A', 0, endian='little')
         self.assertEqual(a.endian(), 'little')
@@ -110,7 +152,6 @@ class CreateObjectTests(unittest.TestCase, Util):
         self.assertRaises(TypeError, bitarray.__new__, bitarray, endian=0)
         self.assertRaises(ValueError, bitarray.__new__, bitarray, endian='')
 
-
     def test_integers(self):
         for n in xrange(50):
             a = bitarray(n)
@@ -124,7 +165,6 @@ class CreateObjectTests(unittest.TestCase, Util):
         self.assertRaises(ValueError, bitarray.__new__, bitarray, -1)
         self.assertRaises(ValueError, bitarray.__new__, bitarray, -924L)
 
-
     def test_list(self):
         lst = ['foo', None, [1], {}]
         a = bitarray(lst)
@@ -137,7 +177,6 @@ class CreateObjectTests(unittest.TestCase, Util):
             self.assertEqual(a.tolist(), lst)
             self.check_obj(a)
 
-
     def test_tuple(self):
         tup = ('', True, [], {1:2})
         a = bitarray(tup)
@@ -150,7 +189,6 @@ class CreateObjectTests(unittest.TestCase, Util):
             self.assertEqual(a.tolist(), lst)
             self.check_obj(a)
         
-
     def test_iter(self):
         for n in xrange(50):
             lst = [bool(randint(0, 1)) for d in xrange(n)]
@@ -167,7 +205,6 @@ class CreateObjectTests(unittest.TestCase, Util):
             self.assertEqual(a, bitarray(lst))
             self.check_obj(a)
 
-
     def test_01(self):
         a = bitarray('0010111')
         self.assertEqual(a.tolist(), [0, 0, 1, 0, 1, 1, 1])
@@ -255,24 +292,26 @@ class MetaDataTests(unittest.TestCase):
     
     def test_buffer_info(self):
         a = bitarray('0000111100001', endian='little')
-        self.assertEqual(a.buffer_info()[1:], (2, 'little', 3))
+        self.assertEqual(a.buffer_info()[1:4], (2, 'little', 3))
         
         a = bitarray()
         self.assertRaises(TypeError, a.buffer_info, 42)
         
         bi = a.buffer_info()
         self.assert_(isinstance(bi, tuple))
-        self.assertEqual(len(bi), 4)
+        self.assertEqual(len(bi), 5)
         
         self.assert_(isinstance(bi[0], (int, long)))
-        self.assert_(isinstance(bi[1], int))
+        self.assert_(isinstance(bi[1], long))
         self.assert_(isinstance(bi[2], str))
         self.assert_(isinstance(bi[3], int))
+        self.assert_(isinstance(bi[4], long))
         
         for n in xrange(50):
             bi = bitarray(n).buffer_info()
             self.assertEqual(bi[1], bits2bytes(n))
-            self.assertEqual(bi[3], 8 * bits2bytes(n) - n)
+            self.assertEqual(bi[3] + n, 8 * bi[1])
+            self.assert_(bi[4] >= bi[1])
         
         a = bitarray(endian='little')
         self.assertEqual(a.buffer_info()[2], 'little')
@@ -289,7 +328,7 @@ class MetaDataTests(unittest.TestCase):
         self.assertEqual(a.endian(), 'big')
 
 
-    def test_len(self):
+    def test_length(self):
         for n in xrange(1000):
             a = bitarray(n)
             self.assertEqual(len(a), n)
@@ -342,7 +381,6 @@ class SliceTests(unittest.TestCase, Util):
                 s = slice(self.rndsliceidx(la),
                           self.rndsliceidx(la), step)
                 self.assertEQUAL(a[s], bitarray(aa[s], endian=a.endian()))
-
         
     def test_setitem(self):
         a = bitarray([False])
@@ -360,6 +398,34 @@ class SliceTests(unittest.TestCase, Util):
         self.assertRaises(IndexError, a.__setitem__,  2, True)
         self.assertRaises(IndexError, a.__setitem__, -3, False)
         
+        for a in self.randombitarrays():
+            la = len(a)
+            if la == 0:
+                continue
+            i = randint(0, la-1)
+            aa = a.tolist()
+            ida = id(a)
+            val = bool(randint(0, 1))
+            a[i] = val
+            aa[i] = val
+            self.assertEqual(a.tolist(), aa)
+            self.assertEqual(id(a), ida)
+            self.check_obj(a)
+
+            b = bitarray(la)
+            b[0:la] = bitarray(a)
+            self.assertEqual(a, b)
+            self.assertNotEqual(id(a), id(b))
+
+            b = bitarray(la)
+            b[:] = bitarray(a)
+            self.assertEqual(a, b)
+            self.assertNotEqual(id(a), id(b))
+
+            b = bitarray(la)
+            b[::-1] = bitarray(a)
+            self.assertEqual(a.tolist()[::-1], b.tolist())
+
         a = bitarray(5*[False])
         a[0] = 1
         a[-2] = 1
@@ -426,7 +492,6 @@ class MiscTests(unittest.TestCase, Util):
         self.assertEqual(bool(bitarray('0')), True)
         self.assertEqual(bool(bitarray('1')), True)
 
-
     def test_iterate(self):
         for lst in self.randomlists():
             acc = []
@@ -434,7 +499,6 @@ class MiscTests(unittest.TestCase, Util):
                 acc.append(b)
             self.assertEqual(acc, lst)
 
-
     def test_asDictKey(self):
         d = {bitarray('011') : 3,
              bitarray('101') : 5,
@@ -444,6 +508,18 @@ class MiscTests(unittest.TestCase, Util):
 
         self.assertEqual(d[bitarray('000')], 0)
 
+    def test_compare(self):
+        for a in self.randombitarrays():
+            aa = a.tolist()
+            
+            for b in self.randombitarrays():
+                bb = b.tolist()
+                self.assertEqual(a == b, aa == bb)
+                self.assertEqual(a != b, aa != bb)
+                self.assertEqual(a <= b, aa <= bb)
+                self.assertEqual(a <  b, aa <  bb)
+                self.assertEqual(a >= b, aa >= bb)
+                self.assertEqual(a >  b, aa >  bb)
 
     def test_subclassing(self):
         class ExaggeratingBitarray(bitarray):
@@ -464,7 +540,6 @@ class MiscTests(unittest.TestCase, Util):
             for i in xrange(len(a)):
                 self.assertEqual(a[i], b[i+1234])
 
-
     def test_endianness(self):
         self.assertEqual(bitarray('\x01', 0, 'little').to01(), '10000000')
         self.assertEqual(bitarray('\x80', 0, 'little').to01(), '00000001')
@@ -508,7 +583,6 @@ class MiscTests(unittest.TestCase, Util):
         self.assertNotEqual(a, b)
         self.assertEqual(a.tostring(), b.tostring())
 
-
     def test_pickle(self):
         from pickle import loads, dumps
         for a in self.randombitarrays():
@@ -516,7 +590,6 @@ class MiscTests(unittest.TestCase, Util):
             self.assert_(b is not a)
             self.assertEQUAL(a, b)
 
-
     def test_cPickle(self):
         from cPickle import loads, dumps
         for a in self.randombitarrays():
@@ -566,6 +639,10 @@ class SpecialMethodTests(unittest.TestCase, Util):
     def test_copy(self):
         import copy
         for a in self.randombitarrays():
+            b = a.copy()
+            self.assert_(b is not a)
+            self.assertEQUAL(a, b)
+            
             b = copy.copy(a)
             self.assert_(b is not a)
             self.assertEQUAL(a, b)
@@ -575,19 +652,11 @@ class SpecialMethodTests(unittest.TestCase, Util):
             self.assertEQUAL(a, b)
 
 
-    def test_compare(self):
-        for a in self.randombitarrays():
-            aa = a.tolist()
-            
-            for b in self.randombitarrays():
-                bb = b.tolist()
-                self.assertEqual(a == b, aa == bb)
-                self.assertEqual(a != b, aa != bb)
-                self.assertEqual(a <= b, aa <= bb)
-                self.assertEqual(a <  b, aa <  bb)
-                self.assertEqual(a >= b, aa >= bb)
-                self.assertEqual(a >  b, aa >  bb)
+tests.append(SpecialMethodTests)
 
+# ---------------------------------------------------------------------------
+
+class NumberTests(unittest.TestCase, Util):
 
     def test_add(self):
         c = bitarray('001') + bitarray('110')
@@ -669,7 +738,91 @@ class SpecialMethodTests(unittest.TestCase, Util):
         self.assertRaises(TypeError, a.__imul__, None)
 
 
-tests.append(SpecialMethodTests)
+tests.append(NumberTests)
+
+# ---------------------------------------------------------------------------
+
+class BitwiseTests(unittest.TestCase, Util):
+
+    def test_misc(self):
+        for a in self.randombitarrays():
+            b = ~a
+            c = a & b
+            self.assertEqual(c.any(), False)
+            self.assertEqual(a, a ^ c)
+            d = a ^ b
+            self.assertEqual(d.all(), True)
+            b &= d
+            self.assertEqual(~b, a)
+    
+    def test_and(self):
+        a = bitarray('11001')
+        b = bitarray('10011')
+        self.assertEQUAL(a & b, bitarray('10001'))
+
+        b = bitarray('1001')
+        self.assertRaises(ValueError, a.__and__, b)
+
+        
+    def test_iand(self):
+        a =  bitarray('110010110')
+        a &= bitarray('100110011')
+        self.assertEQUAL(a, bitarray('100010010'))
+
+    def test_or(self):
+        a = bitarray('11001')
+        b = bitarray('10011')
+        self.assertEQUAL(a | b, bitarray('11011'))
+        
+    def test_iand(self):
+        a =  bitarray('110010110')
+        a |= bitarray('100110011')
+        self.assertEQUAL(a, bitarray('110110111'))
+
+    def test_xor(self):
+        a = bitarray('11001')
+        b = bitarray('10011')
+        self.assertEQUAL(a ^ b, bitarray('01010'))
+        
+    def test_ixor(self):
+        a =  bitarray('110010110')
+        a ^= bitarray('100110011')
+        self.assertEQUAL(a, bitarray('010100101'))
+
+    def test_invert(self):
+        a = bitarray()
+        a.invert()
+        self.assertEQUAL(a, bitarray())
+        
+        a = bitarray('11011')
+        a.invert()
+        self.assertEQUAL(a, bitarray('00100'))
+
+        a = bitarray('11011')
+        b = ~a
+        self.assertEQUAL(b, bitarray('00100'))
+        self.assertEQUAL(a, bitarray('11011'))
+        self.assert_(a is not b)
+        
+        for a in self.randombitarrays():
+            aa = a.tolist()
+            b = bitarray(a)
+            b.invert()
+            for i in xrange(len(a)):
+                self.assertEqual(b[i], not aa[i])
+            self.check_obj(b)
+            
+            c = ~a
+            self.assert_(c is not a)
+            self.assertEQUAL(a, bitarray(aa, endian=a.endian()))
+            
+            for i in xrange(len(a)):
+                self.assertEqual(c[i], not aa[i])
+            
+            self.check_obj(b)
+
+
+tests.append(BitwiseTests)
 
 # ---------------------------------------------------------------------------
 
@@ -708,6 +861,120 @@ tests.append(SequenceTests)
 
 # ---------------------------------------------------------------------------
 
+class ExtendTests(unittest.TestCase, Util):
+
+    def test_wrongArgs(self):
+        a = bitarray()
+        self.assertRaises(TypeError, a.extend)
+        self.assertRaises(TypeError, a.extend, None)
+        self.assertRaises(TypeError, a.extend, True)
+        self.assertRaises(TypeError, a.extend, 24)
+        self.assertRaises(ValueError, a.extend, '0011201')
+    
+    def test_bitarray(self):
+        a = bitarray()
+        a.extend(bitarray())
+        self.assertEqual(a, bitarray())
+        a.extend(bitarray('110'))
+        self.assertEqual(a, bitarray('110'))
+        a.extend(bitarray('1110'))
+        self.assertEqual(a, bitarray('1101110'))
+
+        a = bitarray('00001111', endian='little')
+        a.extend(bitarray('00111100', endian='big'))
+        self.assertEqual(a, bitarray('0000111100111100'))
+        
+        for a in self.randombitarrays():
+            for b in self.randombitarrays():
+                c = bitarray(a)
+                idc = id(c)
+                c.extend(b)
+                self.assertEqual(id(c), idc)
+                self.assertEqual(c, a + b)
+
+    def test_list(self):
+        a = bitarray()
+        a.extend([0, 1, 3, None, {}])
+        self.assertEqual(a, bitarray('01100'))
+        a.extend([True, False])
+        self.assertEqual(a, bitarray('0110010'))
+        
+        for a in self.randomlists():
+            for b in self.randomlists():
+                c = bitarray(a)
+                idc = id(c)
+                c.extend(b)
+                self.assertEqual(id(c), idc)
+                self.assertEqual(c.tolist(), a + b)
+                self.check_obj(c)
+
+    def test_iterable(self):
+        def bar():
+            for x in ('', '1', None, True, []):
+                yield x
+        a = bitarray()
+        a.extend(bar())
+        self.assertEqual(a, bitarray('01010'))
+ 
+        for a in self.randomlists():
+            for b in self.randomlists():
+                def foo():
+                    for e in b:
+                        yield e
+                c = bitarray(a)
+                idc = id(c)
+                c.extend(foo())
+                self.assertEqual(id(c), idc)
+                self.assertEqual(c.tolist(), a + b)
+                self.check_obj(c)
+
+    def test_tuple(self):
+        a = bitarray()
+        a.extend((0, 1, 2, 0, 3))
+        self.assertEqual(a, bitarray('01101'))
+ 
+        for a in self.randomlists():
+            for b in self.randomlists():
+                c = bitarray(a)
+                idc = id(c)
+                c.extend(tuple(b))
+                self.assertEqual(id(c), idc)
+                self.assertEqual(c.tolist(), a + b)
+                self.check_obj(c)
+
+    def test_iter(self):
+        a = bitarray()
+        a.extend(iter([3, 9, 0, 1, -2]))
+        self.assertEqual(a, bitarray('11011'))
+ 
+        for a in self.randomlists():
+            for b in self.randomlists():
+                c = bitarray(a)
+                idc = id(c)
+                c.extend(iter(b))
+                self.assertEqual(id(c), idc)
+                self.assertEqual(c.tolist(), a + b)
+                self.check_obj(c)
+
+    def test_string01(self):
+        a = bitarray()
+        a.extend('0110111')
+        self.assertEqual(a, bitarray('0110111'))
+        
+        for a in self.randomlists():
+            for b in self.randomlists():
+                c = bitarray(a)
+                idc = id(c)
+                c.extend(''.join(('1' if x else '0') for x in b))
+                self.assertEqual(id(c), idc)
+                self.assertEqual(c.tolist(), a + b)
+                self.check_obj(c)
+
+
+tests.append(ExtendTests)
+
+# ---------------------------------------------------------------------------
+
 class MethodTests(unittest.TestCase, Util):
 
     def test_append(self):
@@ -728,83 +995,6 @@ class MethodTests(unittest.TestCase, Util):
             self.assertEQUAL(b, bitarray(aa+[1, 0], endian=a.endian()))
 
 
-    def test_extend(self): # FIXME: This sould be in its own test class
-        a = bitarray()
-        a.extend([-1, None, 1, 0, 2])
-        self.assertEQUAL(a, bitarray('10101'))
-
-        self.assertRaises(TypeError, a.extend)
-        self.assertRaises(TypeError, a.extend, None)
-        self.assertRaises(TypeError, a.extend, True)
-        self.assertRaises(TypeError, a.extend, 24)
-        self.assertRaises(ValueError, a.extend, '0011201')
-        
-        a = bitarray()
-        a.extend(bitarray('01'))
-        a.extend([0, 1])
-        a.extend((0, 1))
-        a.extend('01')
-        self.assertEqual(a, bitarray('01010101'))
-        
-        for a in self.randomlists():
-            for b in self.randombitarrays():
-                c = bitarray(a)
-                idc = id(c)
-                c.extend(b)                             # bitarray
-                self.assertEqual(id(c), idc)
-                self.assertEqual(c, bitarray(a + b.tolist()))
-
-            for b in self.randomlists():
-                c = bitarray(a)
-                idc = id(c)
-                c.extend(b)                               # list
-                self.assertEqual(id(c), idc)
-                self.assertEqual(c.tolist(), a + b)
-                self.check_obj(c)
-
-                def foo():
-                    for e in b:
-                        yield e
-                c = bitarray(a)
-                idc = id(c)
-                c.extend(foo())                          # iterable
-                self.assertEqual(id(c), idc)
-                self.assertEqual(c.tolist(), a + b)
-                self.check_obj(c)
-
-                c = bitarray(a)
-                idc = id(c)
-                c.extend(tuple(b))                          # tuple
-                self.assertEqual(id(c), idc)
-                self.assertEqual(c.tolist(), a + b)
-                self.check_obj(c)
-
-                c = bitarray(a)
-                idc = id(c)
-                c.extend(iter(b))                          # iter
-                self.assertEqual(id(c), idc)
-                self.assertEqual(c.tolist(), a + b)
-                self.check_obj(c)
-
-                c = bitarray(a)
-                idc = id(c)
-                c.extend(''.join(('1' if x else '0')
-                                 for x in b))                 # string01
-                self.assertEqual(id(c), idc)
-                self.assertEqual(c.tolist(), a + b)
-                self.check_obj(c)
-
-                def foo():
-                    for x in b:
-                        yield x
-                c = bitarray(a)
-                idc = id(c)
-                c.extend(foo())                             # iter2
-                self.assertEqual(id(c), idc)
-                self.assertEqual(c.tolist(), a + b)
-                self.check_obj(c)
-
-
     def test_insert(self):
         a = bitarray()
         b = a
@@ -925,7 +1115,7 @@ class MethodTests(unittest.TestCase, Util):
         a.reverse()
         self.assertEQUAL(a, bitarray('11000011111'))
         
-        for a in self.randombitarrays(100):
+        for a in self.randombitarrays():
             aa = a.tolist()
             ida = id(a)
             a.reverse()
@@ -1022,25 +1212,6 @@ class MethodTests(unittest.TestCase, Util):
             self.assert_(a is b)
             self.check_obj(b)
 
-  
-    def test_invert(self):
-        a = bitarray()
-        a.invert()
-        self.assertEQUAL(a, bitarray())
-        
-        a = bitarray('11011')
-        a.invert()
-        self.assertEQUAL(a, bitarray('00100'))
-        
-        for a in self.randombitarrays():
-            aa = a.tolist()
-            b = a
-            b.invert()
-            for i in xrange(len(a)):
-                self.assertEqual(b[i], not aa[i])
-            self.assert_(a is b)
-            self.check_obj(b)
-
 
     def test_bytereverse(self):
         a = bitarray()
@@ -1145,7 +1316,52 @@ class StringTests(unittest.TestCase, Util):
         a.from01('01110');      self.assertEqual(a.to01(), '1001110')
         a.from01('100');        self.assertEqual(a.to01(), '1001110100')
 
+    def test_unpack(self):
+        a = bitarray('01')
+        self.assertEqual(a.unpack(), '\x00\xff')
+        self.assertEqual(a.unpack('A'), 'A\xff')
+        self.assertEqual(a.unpack('0', '1'), '01')
+        self.assertEqual(a.unpack(one='\x01'), '\x00\x01')
+        self.assertEqual(a.unpack(zero='A'), 'A\xff')
+        self.assertEqual(a.unpack(one='t', zero='f'), 'ft')
+        self.assertEqual(a.unpack('a', one='b'), 'ab')
 
+        self.assertRaises(TypeError, a.unpack, 'a', zero='b')
+        self.assertRaises(TypeError, a.unpack, foo='b')
+        self.assertRaises(TypeError, a.unpack, 'a', 'b', 'c')
+        
+        for a in self.randombitarrays():
+            self.assertEqual(a.unpack('0', '1'), a.to01())
+            
+            b = bitarray()
+            b.pack(a.unpack())
+            self.assertEqual(b, a)
+            
+            b = bitarray()
+            b.pack(a.unpack('\x01', '\x00'))
+            b.invert()
+            self.assertEqual(b, a)
+
+    def test_pack(self):
+        a = bitarray()
+        a.pack('\x00')
+        self.assertEqual(a, bitarray('0'))
+        a.pack('\xff')
+        self.assertEqual(a, bitarray('01'))
+        a.pack('\x01\x00\x7a')
+        self.assertEqual(a, bitarray('01101'))
+
+        a = bitarray()
+        for n in xrange(256):
+            a.pack(chr(n))
+        self.assertEqual(a, bitarray('0' + 255 * '1'))
+        
+        self.assertRaises(TypeError, a.pack, 0)
+        self.assertRaises(TypeError, a.pack, u'1')
+        self.assertRaises(TypeError, a.pack, [1, 3])
+        self.assertRaises(TypeError, a.pack, bitarray())
+
+        
 tests.append(StringTests)
 
 # ---------------------------------------------------------------------------
@@ -1311,23 +1527,29 @@ def pages():
     dat = open('/proc/%i/statm' % os.getpid()).read()
     return int(dat.split()[0])
 
-
-def run(verbosity, chk_mem_leaks=False):
+def run(verbosity, chk_mem_leaks=False, times=1):
     suite = unittest.TestSuite()
     for cls in tests:
         suite.addTest(unittest.makeSuite(cls))
     
     runner = unittest.TextTestRunner(verbosity=verbosity)
-    res = runner.run(suite)
+    for dum in xrange(times):
+        res = runner.run(suite)
 
     if chk_mem_leaks:
-        if sys.platform.startswith('linux'):
-            a = pages()
-            for x in xrange(100):
-                runner.run(suite)
-                print 'pages:', pages()
-            b = pages()
-            print 'Pages before and after tests:', a, b
+        if sys.platform == 'linux2':
+            i = 0
+            logfile = 'pages.log'
+            try: os.unlink(logfile)
+            except: pass
+            while True:
+                r = runner.run(suite)
+                p = pages()
+                if i % 100 == 0:
+                    fo = open(logfile, 'a')
+                    fo.write('%10i %r %10i\n' % (i, r.wasSuccessful(), p))
+                    fo.close()
+                i += 1
         else:
             print 'Sorry, can only test for memory leaks on Linux.'
 
@@ -1335,13 +1557,13 @@ def run(verbosity, chk_mem_leaks=False):
 
 
 if __name__ == '__main__':
-    # For development
     arg = ''
     if len(sys.argv)==2:
         arg = sys.argv[1]
     
     run(verbosity     = 2 if 'v' in arg else 1,
         chk_mem_leaks = bool('m' in arg),
+        times         = int(arg) if arg.isdigit() else 1
         )
 
 else:
